@@ -15,7 +15,9 @@ setClass("covQual",
              parUpper = "numeric",           ## upper bound on pars
              par  = "numeric",               ## params values
              parN = "integer",               ## number of par
-             kernParNames  = "character"     ## depending on kernel
+             kernParNames  = "character",    ## depending on kernel
+             ordered = "logical",
+             intAsChar = "logical"
          ),
          validity = function(object){
              if (length(object@kernParNames) != object@parN) {
@@ -36,90 +38,6 @@ setGeneric("checkX",
            function(object, X, ...) standardGeneric("checkX")
            )
 
-
-## **********************************************************************
-## Check that the QUALITATIVE design matrix X is compatible with the
-## covariance object This is quite different form the continuous input
-## case, because one has to check the content of 'X', and not only the
-## input names.
-## **********************************************************************
-
-setMethod("checkX",
-          signature = signature(object = "covQual", X = "matrix"),
-          definition = function(object, X, strict = FALSE, ...){
-              
-              iN <- inputNames(object)
-              
-              if (strict) {
-                  if (length(iN) != ncol(X) || !all(iN == colnames(X)) )
-                      stop("colnames(X) and inpuNames(object) must be identical")
-              }
-              if ( !all(inputNames(object) %in% colnames(X)) )
-                  stop("all the elements of inputNames(object) must be ",
-                       "in colnames(X)")
-              
-              X <- X[ , iN, drop = FALSE]
-              nX <- ncol(X)
-              
-              for (i in (1L:nX)) {
-                  if (!all(X[ , i] %in% 1L:object@nlevels[i])) {
-                      stop("column ", i, " of matrix 'X' (integer) has bad level. ",
-                           "Must be between 1 and ", object@nlevels[i], "." )
-                  }
-              }
-              
-              X 
-          }
-          )
-    
-
-setMethod("checkX",
-          signature = signature(object = "covQual", X = "data.frame"),
-          definition = function(object, X, strict = FALSE, ...){
-              iN <- inputNames(object)
-              if (strict) {
-                  if (length(iN) != ncol(X) || !all(iN == colnames(X)) )
-                      stop("colnames(X) and inpuNames(object) must be identical")
-              }
-              if (!all(inputNames(object) %in% colnames(X)))
-                  stop("all the elements of inputNames(object) must be ",
-                       " in colnames(X)")
-              
-              X <- X[ , iN, drop = FALSE]
-              
-              ## now check the content of the columns
-              nX <- ncol(X)
-              ind.fac <- sapply(X, is.factor)
-              ind.int <- sapply(X, is.integer)
-              
-              if (!all(ind.fac | ind.int)) {
-                  stop("all columns of 'X' must be \"integer\" or \"factor\"")
-              }
-              if (any(ind.fac)) {
-
-                  for (i in (1L:nX)[ind.fac]) {
-                      
-                      if (!all(levels(X[ , i]) %in% object@levels[[i]])) {
-                          stop("the levels of column ", i, " of 'X' (factor) ",
-                               "must match those in 'object'")
-                      }
-                      
-                      levels(X[ , i]) <- object@levels[[i]]
-                  }
-              }
-              if (any(ind.int)) {
-                  for (i in (1L:nX)[ind.int]) {
-                      if (!all(X[ , i] %in% 1L:object@nlevels[i])) {
-                          stop("column ", i, " of data frame 'X' (integer) has bad level. ",
-                               "Must be between 1 and ", object@nlevels[i], "." )
-                      }
-                  }
-              }
-              
-              X
-              
-          }
-      )
 ##  npar method for class "covQual".
 setMethod("npar",
           signature = signature(object = "covQual"),
@@ -169,7 +87,8 @@ setMethod("coef<-",
             object@par[] <- value
             ## replace the slot
             object@covLevMat <- object@covLevels(value)
-            
+            colnames(object@covLevMat) <- rownames(object@covLevMat) <-
+                object@levels[[1]]
             object
           })
 
@@ -181,7 +100,9 @@ setMethod("coef<-",
 setMethod("coefLower", 
           signature = signature(object = "covQual"),
           definition = function(object){
-            object@parLower            
+              res <- object@parLower
+              names(res) <- object@kernParNames
+              res         
           })
 
 setMethod("coefLower<-",
@@ -197,7 +118,9 @@ setMethod("coefLower<-",
 setMethod("coefUpper", 
           signature = signature(object = "covQual"),
           definition = function(object){
-            object@parUpper            
+              res <- object@parUpper
+              names(res) <- object@kernParNames
+              res            
           })
 
 setMethod("coefUpper<-",
@@ -274,10 +197,12 @@ setMethod("show",
             if (object@d == 1L) {
                 if (object@nlevels[1] <= 10) {
                     cat("o Covariance between levels: \n")
-                    print(covMat(object))
+                    M <- covMat(object)
+                    attr(M, "gradient") <- NULL
+                    print(M)
                 } else {
-                    cat("use 'covMat' with only one argument to see the \n",
-                        "covariance matrix.")
+                    cat("use 'covMat' with only one argument to see the ",
+                        "covariance matrix.\n")
                 }
             }       
           })
@@ -288,20 +213,29 @@ setMethod("show",
 setMethod("covMat",
           signature = signature(object = "covQual"), 
           definition = function(object, X, Xnew = NULL,
-              compGrad = FALSE, 
-              checkNames = NULL, lowerSQRT = FALSE,
+              compGrad = hasGrad(object), 
+              checkNames = NULL,
+              intAsChar = object@intAsChar,
+              lowerSQRT = FALSE,
               ...) {
               
+              isXnew <- !is.null(Xnew)
+              if (isXnew) compGrad <- FALSE
+              
               if (missing(X)) {
                   
                   ## this exit was added on 2017-10-11
  
-                  if (!compGrad && !lowerSQRT) {
+                  # if (!compGrad && !lowerSQRT) {
+                  if (!lowerSQRT) {
                       return(object@covLevMat)
                   }
 
                   if (object@d == 1L) {
-                      f <- factor(object@levels[[1]])
+                      ## CAUTION without levels = this would not
+                      ## work as expected for ordinal factors.
+                      f <- factor(seq_along(object@levels[[1]]),
+                                  labels = object@levels[[1]])
                       X <- data.frame(f)
                       colnames(X) <- object@inputNames[1L]
                       rownames(X) <- object@levels[[1]]
@@ -311,24 +245,23 @@ setMethod("covMat",
                   }
               }
               
-              isXnew <- !is.null(Xnew)
               ## X <- as.matrix(X)
               
               if (is.null(checkNames)) {
                   checkNames <- TRUE 
-                  if (object@d == 1L) {
-                      if (ncol(X) == 1L) {
-                          checkNames <- FALSE
-                      }  
-                  }      
+                  # if (object@d == 1L) {
+                  #     if (ncol(X) == 1L) {
+                  #         checkNames <- FALSE
+                  #     }  
+                  # }      
               }
               
-              if (checkNames) X <- checkX(object, X = X)
+              if (checkNames) X <- checkX(object, X = X, intAsChar = intAsChar)
               if (any(is.na(X))) stop("'X' must not contain NA elements")
 
               if (isXnew){
                   ## Xnew <- as.matrix(Xnew)
-                  if (checkNames) Xnew <- checkX(object, X = Xnew)
+                  if (checkNames) Xnew <- checkX(object, X = Xnew, intAsChar = intAsChar)
                   if (ncol(X) != ncol(Xnew)) {
                       stop("'X' and 'Xnew' must have the same number of columns")
                   }
@@ -389,7 +322,9 @@ setMethod("covMat",
 setMethod("varVec",
           signature = signature(object = "covQual"), 
           definition = function(object, X,
-              compGrad = FALSE,  checkNames = NULL,
+              compGrad = FALSE,
+              checkNames = NULL,
+              intAsChar = object@intAsChar,
               ...) {
               
               if (missing(X)) {
@@ -411,7 +346,7 @@ setMethod("varVec",
                   }      
               }
               
-              if (checkNames) X <- checkX(object, X = X)
+              if (checkNames) X <- checkX(object, X = X, intAsChar = intAsChar)
               if (any(is.na(X))) stop("'X' must not contain NA elements")
               
               compGrad <- as.integer(compGrad)
@@ -494,3 +429,159 @@ setMethod("simulate",
               ## attr(val, "seed") <- RNGstate
               val
           })
+
+
+
+## *************************************************************************
+## plot a covariance structure.
+## *************************************************************************
+
+plot.covQual <- function(x, type = c("cov", "cor", "warping"), ...){
+  type <- match.arg(type)
+  if (type %in% c("cor", "cov")){
+    C <- covMat(x)
+    is.corr <- (type == "cor")
+    if (type == "cor") C <- cov2cor(C)
+    if (requireNamespace("corrplot", quietly = TRUE)){
+      corrplot::corrplot(C, is.corr = is.corr, ...)
+    } else {
+      levelplot(C, ...)
+      # image(C[nrow(C):1, nrow(C):1], xaxt = "n", yaxt = "n", ...)
+      # axis(side = 3, at = seq(0, 1, length.out = x@nlevels), labels = x@levels[[1]])
+      # axis(side = 2, at = seq(0, 1, length.out = x@nlevels), labels = rev(x@levels[[1]]))
+    }
+  }
+  if (type == "warping"){
+    if (class(x) != "covOrd"){
+      warning("Type 'warping' only works for an ordinal factor")
+      return(NULL)
+    }
+    parWarp <- coef(x)[1:x@parNwarp]
+    zseq <- seq(0, 1, length.out = 200)
+    yseq <- x@warpFun$fun(zseq, par = parWarp, L = x@nlevels)
+    plot(zseq, yseq, type = "l", xaxt = "n", 
+         xlab = inputNames(x), ylab = "", ...)
+    levseq <- seq(0, 1, length.out = x@nlevels)
+    axis(side = 1, at = levseq, labels = x@levels[[1]])
+    points(levseq, x@warpFun$fun(levseq, par = parWarp, L = x@nlevels), pch = 19)
+    if (x@parNk1 > 0) lines(c(0, 1), c(0, 1), lty = "dotted")  # in that case, F is normalized
+  }
+}
+
+if(!isGeneric("plot")) {
+  setGeneric(name = "plot",
+             def = function(x, y, ...) standardGeneric("plot")
+  )
+}
+
+setMethod("plot",
+          signature(x = "covQual"), 
+          function(x, y, type = c("cov", "cor", "warping"), ...){
+            if (!missing(y)) warning("Argument y is ignored (not used in this plot method)")
+            plot.covQual(x = x, type = type, ...)
+          }
+          )
+
+## **********************************************************************
+## Check that the QUALITATIVE design matrix X is compatible with the
+## covariance object This is quite different form the continuous input
+## case, because one has to check the content of 'X', and not only the
+## input names.
+## **********************************************************************
+
+setMethod("checkX",
+          signature = signature(object = "covQual", X = "matrix"),
+          definition = function(object, X, strict = FALSE, ...){
+              
+              iN <- inputNames(object)
+              
+              if (strict) {
+                  if (length(iN) != ncol(X) || !all(iN == colnames(X)) )
+                      stop("colnames(X) and inpuNames(object) must be identical")
+              }
+              if ( !all(inputNames(object) %in% colnames(X)) )
+                  stop("all the elements of inputNames(object) must be ",
+                       "in colnames(X)")
+              
+              X <- X[ , iN, drop = FALSE]
+              nX <- ncol(X)
+
+              checkX(object = object, X = as.data.frame(X))
+
+          }
+          )
+    
+
+setMethod("checkX",
+         signature = signature(object = "covQual", X = "data.frame"),
+         definition = function(object, X, strict = FALSE, intAsChar = object@intAsChar, ...){
+             iN <- inputNames(object)
+             if (strict) {
+                 if (length(iN) != ncol(X) || !all(iN == colnames(X)) )
+                     stop("colnames(X) and inpuNames(object) must be identical")
+             }
+             if ( !all(inputNames(object) %in% colnames(X)) )
+                 stop("all the elements of inputNames(object) must be",
+                      " in colnames(X)")
+             u <- X[ , iN]
+             levs <- object@levels[[1]]
+
+             if (is.integer(u) && intAsChar) {
+                 u <- as.character(u)
+             }
+             
+             if (is.integer(u)) {
+                 if (any(u < 1) || any(u > object@nlevels)) {
+                     stop("When 'inputNames(object)' corresponds to an ",
+                          "integer column in 'X', this must contain values ",
+                          "between 1 and the number of levels of 'object'")
+                 }
+                 if (object@ordered[1]) {  # works only for one factor
+                   u <- ordered(levs[u], levels = levs)
+                   warning("Automatic coercion used when input levels are integer") 
+                   #         The first observations are: ")
+                   # print(u[1:min(4, nlevels(object)[1]])
+                 } else {
+                   u <- factor(levs[u], levels = levs)
+                 }
+             } else if (is.character(u)) {
+                 if (!all(u %in% levs)) {
+                     stop("When 'inputNames(object)' corresponds to a ",
+                          "character column in 'X', this must contain only ",
+                          "values found in the levels of 'object'")
+                 }
+                 names(levs) <- levs
+                 if (object@ordered[1]) {  # works only for one factor
+                    u <- ordered(levs[u], levels = levs)
+                 } else {
+                    u <- factor(levs[u], levels = levs)
+                 }
+               
+             }  else if (is.factor(u)) {
+
+                 levsu <- levels(u)
+                 if (!identical(levsu, levs)) {        
+                     if (!setequal(levsu, levs)) {
+                         stop("When 'inputNames(object)' corresponds to a ",
+                              "factor column in 'X', the levels must be identical ",
+                              "to the levels of 'object'")
+                     } else {
+                         warning("The levels of the factor input are not in the same ",
+                                 "order as that of the 'covQual' object. Forcing the same ",
+                                 "order")
+                         u <- factor(u, levels = levs)
+                     }
+                 }
+                 
+                 if (object@ordered[1]) {  # works only for one factor
+                   u <- as.ordered(u)
+                 } 
+                 
+             } else  {
+                 stop("'inputNames(object)' must correspond to a column in 'X' ",
+                      "which is 'integer', 'character' or 'factor'")
+             }
+             df <- data.frame(u)
+             colnames(df) <- iN
+             df
+           })
